@@ -10,36 +10,30 @@ except NameError:
 
 import sys
 import warnings
-import inspect
 
 import sqlalchemy
-from sqlalchemy                     import Table, Column, Integer, String, \
-                                           desc, ForeignKey, and_
-from sqlalchemy.orm                 import Query, MapperExtension,\
-                                           mapper, object_session, EXT_PASS
-from sqlalchemy.ext.sessioncontext  import SessionContext
+from sqlalchemy                    import Table, Column, Integer, \
+                                          desc, ForeignKey, and_
+from sqlalchemy.orm                import Query, MapperExtension,\
+                                          mapper, object_session, EXT_PASS
+from sqlalchemy.ext.sessioncontext import SessionContext
 
 import elixir
-from elixir.statements              import process_mutators
-from elixir.options                 import options_defaults, valid_options
-from elixir.properties              import Property
+from elixir.statements import process_mutators
+from elixir import options
+from elixir.properties import Property
 
 
 __doc_all__ = ['Entity', 'EntityMeta']
 
-DEFAULT_AUTO_PRIMARYKEY_NAME = "id"
-DEFAULT_AUTO_PRIMARYKEY_TYPE = Integer
-DEFAULT_VERSION_ID_COL = "row_version"
-DEFAULT_POLYMORPHIC_COL_NAME = "row_type"
-DEFAULT_POLYMORPHIC_COL_SIZE = 40
-DEFAULT_POLYMORPHIC_COL_TYPE = String(DEFAULT_POLYMORPHIC_COL_SIZE)
 
 try: 
     from sqlalchemy.orm import ScopedSession
 except ImportError: 
     # Not on sqlalchemy version 0.4
     ScopedSession = type(None)
-    
+
+
 def _do_mapping(session, cls, *args, **kwargs):
     if session is None:
         return mapper(cls, *args, **kwargs)
@@ -57,6 +51,7 @@ def _do_mapping(session, cls, *args, **kwargs):
         class query(object):
             def __getattr__(s, key):
                 return getattr(session.registry().query(cls), key)
+
             def __call__(s):
                 return session.registry().query(cls)
 
@@ -86,11 +81,12 @@ class EntityDescriptor(object):
 
         self.builders = []
 
+        self.is_base = is_base(entity)
         self.parent = None
         self.children = []
 
         for base in entity.__bases__:
-            if issubclass(base, Entity) and base is not Entity:
+            if isinstance(base, EntityMeta) and not is_base(base):
                 if self.parent:
                     raise Exception('%s entity inherits from several entities,'
                                     ' and this is not supported.' 
@@ -123,10 +119,11 @@ class EntityDescriptor(object):
                        'autoload', 'tablename', 'shortnames', 
                        'auto_primarykey', 'version_id_col', 
                        'allowcoloverride'):
-            setattr(self, option, options_defaults[option])
+            setattr(self, option, options.options_defaults[option])
 
         for option_dict in ('mapper_options', 'table_options'):
-            setattr(self, option_dict, options_defaults[option_dict].copy())
+            setattr(self, option_dict, 
+                    options.options_defaults[option_dict].copy())
 
     def setup_options(self):
         '''
@@ -182,47 +179,49 @@ class EntityDescriptor(object):
 
     def create_pk_cols(self):
         """
-        Create primary_key columns. That is, add columns from belongs_to
-        relationships marked as being a primary_key and then add a primary 
-        key to the table if it hasn't already got one and needs one. 
+        Create primary_key columns. That is, call the 'create_pk_cols' 
+        builders then add a primary key to the table if it hasn't already got 
+        one and needs one. 
         
-        This method is "semi-recursive" in that it calls the create_keys 
-        method on BelongsTo relationships and those in turn call create_pk_cols
-        on their target. It shouldn't be possible to have an infinite loop 
-        since a loop of primary_keys is not a valid situation.
+        This method is "semi-recursive" in some cases: it calls the 
+        create_keys method on ManyToOne relationships and those in turn call
+        create_pk_cols on their target. It shouldn't be possible to have an 
+        infinite loop since a loop of primary_keys is not a valid situation.
         """
         if self._pk_col_done:
             return
 
-        if self.autoload:
-            return
-
         self.call_builders('create_pk_cols')
 
-        if self.parent:
-            if self.inheritance == 'multi':
-                # add columns with foreign keys to the parent's primary key 
-                # columns 
-                parent_desc = self.parent._descriptor
-                for pk_col in parent_desc.primary_keys:
-                    colname = "%s_%s" % (self.parent.__name__.lower(),
-                                         pk_col.key)
+        if not self.autoload:
+            if self.parent:
+                if self.inheritance == 'multi':
+                    # add columns with foreign keys to the parent's primary 
+                    # key columns 
+                    parent_desc = self.parent._descriptor
+                    for pk_col in parent_desc.primary_keys:
+                        colname = options.MULTIINHERITANCECOL_NAMEFORMAT % \
+                                  {'entity': self.parent.__name__.lower(),
+                                   'key': pk_col.key}
 
-                    # it seems like SA ForeignKey is not happy being given a 
-                    # real column object when said column is not yet attached 
-                    # to a table
-                    pk_col_name = "%s.%s" % (parent_desc.tablename, pk_col.key)
-                    col = Column(colname, pk_col.type, 
-                                 ForeignKey(pk_col_name), primary_key=True)
-                    self.add_column(col)
-        elif not self.has_pk and self.auto_primarykey:
-            if isinstance(self.auto_primarykey, basestring):
-                colname = self.auto_primarykey
-            else:
-                colname = DEFAULT_AUTO_PRIMARYKEY_NAME
-            
-            self.add_column(Column(colname, DEFAULT_AUTO_PRIMARYKEY_TYPE, 
-                                   primary_key=True))
+                        # it seems like SA ForeignKey is not happy being given
+                        # a real column object when said column is not yet 
+                        # attached to a table
+                        pk_col_name = "%s.%s" % (parent_desc.tablename, 
+                                                 pk_col.key)
+                        fk = ForeignKey(pk_col_name, ondelete='cascade')
+                        col = Column(colname, pk_col.type, fk,
+                                     primary_key=True)
+                        self.add_column(col)
+            elif not self.has_pk and self.auto_primarykey:
+                if isinstance(self.auto_primarykey, basestring):
+                    colname = self.auto_primarykey
+                else:
+                    colname = options.DEFAULT_AUTO_PRIMARYKEY_NAME
+                
+                self.add_column(
+                    Column(colname, options.DEFAULT_AUTO_PRIMARYKEY_TYPE, 
+                           primary_key=True))
         self._pk_col_done = True
 
     def setup_relkeys(self):
@@ -255,24 +254,27 @@ class EntityDescriptor(object):
                 for constraint in self.constraints:
                     self.parent._descriptor.add_constraint(constraint)
                 return
-            elif self.inheritance == 'concrete':
-               # copy all columns from parent table
-               for col in self.parent._descriptor.columns:
+            elif self.inheritance == 'concrete': 
+                #TODO: we should also copy columns from the parent table if the
+                # parent is a base entity (whatever the inheritance type -> elif
+                # will need to be changed)
+                # copy all columns from parent table
+                for col in self.parent._descriptor.columns:
                     self.add_column(col.copy())
-               #FIXME: copy constraints. But those are not as simple to copy
-               #since the source column must be changed
+                #FIXME: copy constraints. But those are not as simple to copy
+                #since the source column must be changed
 
         if self.polymorphic and self.inheritance in ('single', 'multi') and \
            self.children and not self.parent:
             if not isinstance(self.polymorphic, basestring):
-                self.polymorphic = DEFAULT_POLYMORPHIC_COL_NAME
+                self.polymorphic = options.DEFAULT_POLYMORPHIC_COL_NAME
                 
             self.add_column(Column(self.polymorphic, 
-                                   DEFAULT_POLYMORPHIC_COL_TYPE))
+                                   options.POLYMORPHIC_COL_TYPE))
 
         if self.version_id_col:
             if not isinstance(self.version_id_col, basestring):
-                self.version_id_col = DEFAULT_VERSION_ID_COL
+                self.version_id_col = options.DEFAULT_VERSION_ID_COL_NAME
             self.add_column(Column(self.version_id_col, Integer))
 
         # create list of columns and constraints
@@ -314,12 +316,11 @@ class EntityDescriptor(object):
 
         # create a list of callbacks for each event
         methods = {}
-        for name, func in inspect.getmembers(self.entity, inspect.ismethod):
-            if hasattr(func, '_elixir_events'):
-                for event in func._elixir_events:
+        for name, method in self.entity.__dict__.items():
+            if hasattr(method, '_elixir_events'):
+                for event in method._elixir_events:
                     event_methods = methods.setdefault(event, [])
-                    event_methods.append(func)
-        
+                    event_methods.append(method)
         if not methods:
             return
         
@@ -397,8 +398,9 @@ class EntityDescriptor(object):
                 if self.children or self.parent:
                     #TODO: make this customizable (both callable and string)
                     #TODO: include module name
-                    kwargs['polymorphic_identity'] = \
-                        self.entity.__name__.lower()
+                    if 'polymorphic_identity' not in kwargs:
+                        kwargs['polymorphic_identity'] = \
+                            self.entity.__name__.lower()
 
                 if self.inheritance == 'concrete':
                     kwargs['concrete'] = True
@@ -442,7 +444,7 @@ class EntityDescriptor(object):
         if check_duplicate is None:
             check_duplicate = not self.allowcoloverride
         
-        if check_duplicate and self.get_column(col.key) is not None:
+        if check_duplicate and self.get_column(col.key, False) is not None:
             raise Exception("Column '%s' already exist in '%s' ! " % 
                             (col.key, self.entity.__name__))
         self._columns.append(col)
@@ -485,12 +487,15 @@ class EntityDescriptor(object):
         extensions.append(extension)
         self.mapper_options['extension'] = extensions
 
-    def get_column(self, key):
+    def get_column(self, key, check_missing=True):
         "need to support both the case where the table is already setup or not"
         #TODO: support SA table/autoloaded entity
         for col in self.columns:
             if col.key == key:
                 return col
+        if check_missing:
+            raise Exception("No column named '%s' found in the table of the "
+                            "'%s' entity!" % (key, self.entity.__name__))
         return None
 
     def get_inverse_relation(self, rel, reverse=False):
@@ -524,17 +529,20 @@ class EntityDescriptor(object):
             if rel.name == name:
                 return rel
         if self.parent:
-            return self.parent.find_relationship(name)
+            return self.parent._descriptor.find_relationship(name)
         else:
             return None
 
     def columns(self):
-        #FIXME: this would be more correct but it breaks inheritance, so I'll use the
-        # old test for now.
+        #FIXME: this would be more correct but it breaks inheritance, so I'll 
+        # use the old test for now.
 #        if self.entity.table:
         if self.autoload: 
             return self.entity.table.columns
         else:
+            #FIXME: depending on the type of inheritance, we should also 
+            # return the parent entity's columns (for example for order_by 
+            # using a column defined in the parent.
             return self._columns
     columns = property(columns)
 
@@ -572,7 +580,9 @@ class TriggerProxy(object):
         proxied_attr = getattr(self.class_, self.attrname)
         return "<TriggerProxy (%s)>" % (self.class_.__name__)
 
+
 class TriggerAttribute(object):
+
     def __init__(self, attrname):
         self.attrname = attrname
 
@@ -583,21 +593,31 @@ class TriggerAttribute(object):
         elixir.setup_all()
         return getattr(owner, self.attrname)
 
-
-def _is_entity(class_):
-    return isinstance(class_, EntityMeta)
+def is_base(cls):
+    """
+    Scan bases classes to see if any is an instance of EntityMeta. If we
+    don't find any, it means the current entity is a base class (like 
+    the 'Entity' class).
+    """
+    for base in cls.__bases__:
+        if isinstance(base, EntityMeta):
+            return False
+    return True
 
 class EntityMeta(type):
     """
     Entity meta class. 
-    You should only use this if you want to define your own base class for your
-    entities (ie you don't want to use the provided 'Entity' class).
+    You should only use it directly if you want to define your own base class 
+    for your entities (ie you don't want to use the provided 'Entity' class).
     """
     _entities = {}
 
     def __init__(cls, name, bases, dict_):
-        # only process subclasses of Entity, not Entity itself
-        if bases[0] is object:
+        # Only process further subclasses of the base classes (Entity et al.),
+        # not the base classes themselves. We don't want the base entities to 
+        # be registered in an entity collection, nor to have a table name and 
+        # so on. 
+        if is_base(cls):
             return
 
         # build a dict of entities for each frame where there are entities
@@ -611,7 +631,7 @@ class EntityMeta(type):
         # will find more entities only if some of them where imported from 
         # another module.
         for entity in [e for e in caller_frame.f_locals.values() 
-                         if _is_entity(e)]:
+                         if isinstance(e, EntityMeta)]:
             caller_entities[entity.__name__] = entity
 
         # create the entity descriptor
@@ -626,7 +646,8 @@ class EntityMeta(type):
         for name, prop in sorted_props:
             prop.attach(cls, name)
 
-        # process mutators. Needed before setup_proxy for metadata
+        # Process mutators. Needed before _install_autosetup_triggers so that
+        # we know of the metadata
         process_mutators(cls)
 
         # setup misc options here (like tablename etc.)
@@ -648,16 +669,12 @@ class EntityMeta(type):
 def _install_autosetup_triggers(cls, entity_name=None):
     #TODO: move as much as possible of those "_private" values to the
     # descriptor, so that we don't mess the initial class.
-    cls._class_key = sqlalchemy.orm.mapperlib.ClassKey(cls, entity_name)
-
     tablename = cls._descriptor.tablename
     schema = cls._descriptor.table_options.get('schema', None)
     cls._table_key = sqlalchemy.schema._get_table_key(tablename, schema)
 
-    mapper_proxy = TriggerProxy(cls, 'mapper')
     table_proxy = TriggerProxy(cls, 'table')
 
-    sqlalchemy.orm.mapper_registry[cls._class_key] = mapper_proxy
     md = cls._descriptor.metadata
     md.tables[cls._table_key] = table_proxy
 
@@ -701,8 +718,6 @@ def _cleanup_autosetup_triggers(cls):
     for name in ('c', 'query'):
         delattr(cls, name)
 
-    del sqlalchemy.orm.mapper_registry[cls._class_key]
-
     desc = cls._descriptor
     md = desc.metadata
 
@@ -721,7 +736,7 @@ def _cleanup_autosetup_triggers(cls):
 def setup_entities(entities):
     '''Setup all entities in the list passed as argument'''
 
-    for entity in elixir.entities:
+    for entity in entities:
         if entity._descriptor.autosetup:
             _cleanup_autosetup_triggers(entity)
 
@@ -729,8 +744,8 @@ def setup_entities(entities):
             'setup_autoload_table', 'create_pk_cols', 'setup_relkeys',
             'before_table', 'setup_table', 'setup_reltables', 'after_table',
             'setup_events',
-            'setup_properties',
             'before_mapper', 'setup_mapper', 'after_mapper',
+            'setup_properties',
             'finalize'):
         for entity in entities:
             if hasattr(entity, '_setup_done'):
@@ -738,6 +753,7 @@ def setup_entities(entities):
             method = getattr(entity._descriptor, method_name)
             method()
 
+
 def cleanup_entities(entities):
     """
     Try to revert back the list of entities passed as argument to the state 
@@ -793,11 +809,15 @@ class Entity(object):
     tutorial.
     '''
     __metaclass__ = EntityMeta
-
+    
     def __init__(self, **kwargs):
         for key, value in kwargs.items():
             setattr(self, key, value)
 
+    def set(self, **kwargs):
+        for key, value in kwargs.items():
+            setattr(self, key, value)
+
     # session methods
     def flush(self, *args, **kwargs):
         return object_session(self).flush([self], *args, **kwargs)
